"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const get_added_keys_from_objects_1 = require("./common/get-added-keys-from-objects");
const get_common_keys_from_objects_1 = require("./common/get-common-keys-from-objects");
const get_removed_keys_from_objects_1 = require("./common/get-removed-keys-from-objects");
const create_difference_1 = require("./create-difference");
const find_diffs_in_operations_1 = require("./find-diffs-in-operations");
const normalize_path_1 = require("./normalize-path");
const findAddedPathDifferences = (sourcePathItems, destinationPathItems) => {
    return get_added_keys_from_objects_1.getAddedKeysFromObjects(sourcePathItems, destinationPathItems)
        .map((addedPathName) => {
        const addedDestinationPathItem = destinationPathItems[addedPathName];
        return create_difference_1.createDifference({
            action: 'add',
            destinationObject: addedDestinationPathItem.originalValue,
            propertyName: 'path'
        });
    });
};
const findRemovedPathDifferences = (sourcePathItems, destinationPathItems) => {
    return get_removed_keys_from_objects_1.getRemovedKeysFromObjects(sourcePathItems, destinationPathItems)
        .map((removedPathName) => {
        const removedSourcePathItem = sourcePathItems[removedPathName];
        return create_difference_1.createDifference({
            action: 'remove',
            propertyName: 'path',
            sourceObject: removedSourcePathItem.originalValue
        });
    });
};
const findMatchingPathDifferences = (sourcePathItems, destinationPathItems) => {
    const matchingPaths = get_common_keys_from_objects_1.getCommonKeysFromObjects(sourcePathItems, destinationPathItems);
    return matchingPaths.reduce((allDifferences, matchingPathItem) => {
        const differencesInOperations = find_diffs_in_operations_1.findDifferencesInOperations(sourcePathItems[matchingPathItem].operations, destinationPathItems[matchingPathItem].operations);
        return [...allDifferences, ...differencesInOperations];
    }, []);
};
const normalizePathItems = (parsedPathItems) => Object.keys(parsedPathItems).reduce((normalizedParsedPathItems, pathName) => {
    const parsedPathItem = parsedPathItems[pathName];
    const normalizedPathName = normalize_path_1.normalizePath(pathName);
    normalizedParsedPathItems[normalizedPathName] = Object.assign({}, parsedPathItem, { pathName: normalizedPathName });
    return normalizedParsedPathItems;
}, {});
exports.findDiffsInPaths = (sourcePathItems, destinationPathItems) => {
    const normalizedSourcePathItems = normalizePathItems(sourcePathItems);
    const normalizedDestinationPathItems = normalizePathItems(destinationPathItems);
    const addedPaths = findAddedPathDifferences(normalizedSourcePathItems, normalizedDestinationPathItems);
    const removedPaths = findRemovedPathDifferences(normalizedSourcePathItems, normalizedDestinationPathItems);
    const matchingPaths = findMatchingPathDifferences(normalizedSourcePathItems, normalizedDestinationPathItems);
    return [...addedPaths, ...removedPaths, ...matchingPaths];
};
