"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const parse_x_properties_1 = require("./common/parse-x-properties");
const typeCheckedOpenApi3Methods = {
    delete: undefined,
    get: undefined,
    head: undefined,
    options: undefined,
    patch: undefined,
    post: undefined,
    put: undefined,
    trace: undefined
};
const isOpenApi3Method = (propertyName) => Object.keys(typeCheckedOpenApi3Methods).indexOf(propertyName) >= 0;
const parseOperations = (pathItemObject, pathItemOriginalPath) => {
    return Object.keys(pathItemObject)
        .filter(isOpenApi3Method)
        .reduce((accumulator, method) => {
        accumulator[method] = {
            originalValue: {
                originalPath: [...pathItemOriginalPath, method],
                value: pathItemObject[method]
            }
        };
        return accumulator;
    }, {});
};
const parsePaths = (paths) => Object.keys(paths).reduce((accumulator, pathName) => {
    const pathItemObject = paths[pathName];
    const originalPath = ['paths', pathName];
    accumulator[pathName] = {
        operations: parseOperations(pathItemObject, originalPath),
        originalValue: {
            originalPath,
            value: pathItemObject
        },
        pathName
    };
    return accumulator;
}, {});
const validateOpenApi3 = (openApi3Spec) => openApi3Spec;
exports.parseOpenApi3Spec = (openApi3Spec) => {
    const validatedOpenApi3Spec = validateOpenApi3(openApi3Spec);
    return {
        format: 'openapi3',
        paths: parsePaths(validatedOpenApi3Spec.paths),
        xProperties: parse_x_properties_1.parseXPropertiesInObject(validatedOpenApi3Spec)
    };
};
